/* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
 * Copyright (C) 2008 - 2012 Magnus Loef <magnus-swe@telia.com> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
*/


#include "../config.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "gettext.h"
#include "widgets.h"
#include "clear_log_tab.h"
#include "populate_log_tab.h"

extern gchar *selected_connection;


/* Clears the VPN client log */
void clear_log_tab(struct w *widgets)
{
    FILE *fp;
    gchar *log_path = g_strdup_printf("%s/log/gadmin-openvpn/client/%s/openvpn-client.log",
                                                         LOCALSTATEDIR, selected_connection);

    if((fp=fopen(log_path, "w+"))==NULL)
    {
        g_free(log_path);
        return;
    }
    fclose(fp);
    g_free(log_path);

    populate_log_tab(widgets);
}
